@@ -1,13 +1,14 @@
 """Constants used by esphome."""
 
-__version__ = "2022.8.3"
+__version__ = "2022.11.0"
 
 ALLOWED_NAME_CHARS = "abcdefghijklmnopqrstuvwxyz0123456789-_"
 
 PLATFORM_ESP32 = "esp32"
 PLATFORM_ESP8266 = "esp8266"
+PLATFORM_RP2040 = "rp2040"
 
-TARGET_PLATFORMS = [PLATFORM_ESP32, PLATFORM_ESP8266]
+TARGET_PLATFORMS = [PLATFORM_ESP32, PLATFORM_ESP8266, PLATFORM_RP2040]
 
 SOURCE_FILE_EXTENSIONS = {".cpp", ".hpp", ".h", ".c", ".tcc", ".ino"}
 HEADER_FILE_EXTENSIONS = {".h", ".hpp", ".tcc"}
@@ -23,6 +24,7 @@ CONF_ACCURACY = "accuracy"
 CONF_ACCURACY_DECIMALS = "accuracy_decimals"
 CONF_ACTION_ID = "action_id"
 CONF_ACTION_STATE_TOPIC = "action_state_topic"
+CONF_ACTIVE = "active"
 CONF_ACTIVE_POWER = "active_power"
 CONF_ADDRESS = "address"
 CONF_ADDRESSABLE_LIGHT_ID = "addressable_light_id"
@@ -114,6 +116,7 @@ CONF_COMMAND_RETAIN = "command_retain"
 CONF_COMMAND_TOPIC = "command_topic"
 CONF_COMMENT = "comment"
 CONF_COMMIT = "commit"
+CONF_COMPILE_PROCESS_LIMIT = "compile_process_limit"
 CONF_COMPONENT_ID = "component_id"
 CONF_COMPONENTS = "components"
 CONF_CONDITION = "condition"
@@ -173,6 +176,7 @@ CONF_DIR_PIN = "dir_pin"
 CONF_DIRECTION = "direction"
 CONF_DIRECTION_OUTPUT = "direction_output"
 CONF_DISABLED_BY_DEFAULT = "disabled_by_default"
+CONF_DISCONNECT_DELAY = "disconnect_delay"
 CONF_DISCOVERY = "discovery"
 CONF_DISCOVERY_OBJECT_ID_GENERATOR = "discovery_object_id_generator"
 CONF_DISCOVERY_PREFIX = "discovery_prefix"
@@ -191,13 +195,16 @@ CONF_DUMMY_RECEIVER_ID = "dummy_receiver_id"
 CONF_DUMP = "dump"
 CONF_DURATION = "duration"
 CONF_EAP = "eap"
+CONF_EC = "ec"
 CONF_ECHO_PIN = "echo_pin"
 CONF_ECO2 = "eco2"
 CONF_EFFECT = "effect"
 CONF_EFFECTS = "effects"
 CONF_ELSE = "else"
+CONF_ENABLE_BTM = "enable_btm"
 CONF_ENABLE_IPV6 = "enable_ipv6"
 CONF_ENABLE_PIN = "enable_pin"
+CONF_ENABLE_RRM = "enable_rrm"
 CONF_ENABLE_TIME = "enable_time"
 CONF_ENERGY = "energy"
 CONF_ENTITY_CATEGORY = "entity_category"
@@ -330,6 +337,7 @@ CONF_LAMBDA = "lambda"
 CONF_LAST_CONFIDENCE = "last_confidence"
 CONF_LAST_FINGER_ID = "last_finger_id"
 CONF_LATITUDE = "latitude"
+CONF_LED = "led"
 CONF_LEGEND = "legend"
 CONF_LENGTH = "length"
 CONF_LEVEL = "level"
@@ -390,6 +398,7 @@ CONF_MIN_POWER = "min_power"
 CONF_MIN_RANGE = "min_range"
 CONF_MIN_TEMPERATURE = "min_temperature"
 CONF_MIN_VALUE = "min_value"
+CONF_MIN_VERSION = "min_version"
 CONF_MINUTE = "minute"
 CONF_MINUTES = "minutes"
 CONF_MISO_PIN = "miso_pin"
@@ -482,12 +491,14 @@ CONF_PACKAGES = "packages"
 CONF_PAGE_ID = "page_id"
 CONF_PAGES = "pages"
 CONF_PANASONIC = "panasonic"
+CONF_PARAMETERS = "parameters"
 CONF_PASSWORD = "password"
 CONF_PATH = "path"
 CONF_PAYLOAD = "payload"
 CONF_PAYLOAD_AVAILABLE = "payload_available"
 CONF_PAYLOAD_NOT_AVAILABLE = "payload_not_available"
 CONF_PERIOD = "period"
+CONF_PH = "ph"
 CONF_PHASE_ANGLE = "phase_angle"
 CONF_PHASE_BALANCER = "phase_balancer"
 CONF_PIN = "pin"
@@ -555,6 +566,7 @@ CONF_RAW_DATA_ID = "raw_data_id"
 CONF_RC_CODE_1 = "rc_code_1"
 CONF_RC_CODE_2 = "rc_code_2"
 CONF_REACTIVE_POWER = "reactive_power"
+CONF_READ_PIN = "read_pin"
 CONF_REBOOT_TIMEOUT = "reboot_timeout"
 CONF_RECEIVE_TIMEOUT = "receive_timeout"
 CONF_RED = "red"
@@ -562,6 +574,7 @@ CONF_REF = "ref"
 CONF_REFERENCE_RESISTANCE = "reference_resistance"
 CONF_REFERENCE_TEMPERATURE = "reference_temperature"
 CONF_REFRESH = "refresh"
+CONF_RELABEL = "relabel"
 CONF_REPEAT = "repeat"
 CONF_REPOSITORY = "repository"
 CONF_RESET_DURATION = "reset_duration"
@@ -647,6 +660,7 @@ CONF_STATE_CLASS = "state_class"
 CONF_STATE_TOPIC = "state_topic"
 CONF_STATIC_IP = "static_ip"
 CONF_STATUS = "status"
+CONF_STB_PIN = "stb_pin"
 CONF_STEP = "step"
 CONF_STEP_MODE = "step_mode"
 CONF_STEP_PIN = "step_pin"
@@ -655,6 +669,7 @@ CONF_STOP_ACTION = "stop_action"
 CONF_STORE_BASELINE = "store_baseline"
 CONF_SUBNET = "subnet"
 CONF_SUBSTITUTIONS = "substitutions"
+CONF_SUM = "sum"
 CONF_SUPPLEMENTAL_COOLING_ACTION = "supplemental_cooling_action"
 CONF_SUPPLEMENTAL_COOLING_DELTA = "supplemental_cooling_delta"
 CONF_SUPPLEMENTAL_HEATING_ACTION = "supplemental_heating_action"
@@ -736,6 +751,7 @@ CONF_USE_ABBREVIATIONS = "use_abbreviations"
 CONF_USE_ADDRESS = "use_address"
 CONF_USERNAME = "username"
 CONF_UUID = "uuid"
+CONF_VALIDITY_PERIOD = "validity_period"
 CONF_VALUE = "value"
 CONF_VALUE_FONT = "value_font"
 CONF_VARIABLES = "variables"
@@ -763,6 +779,7 @@ CONF_WILL_MESSAGE = "will_message"
 CONF_WIND_DIRECTION_DEGREES = "wind_direction_degrees"
 CONF_WIND_SPEED = "wind_speed"
 CONF_WINDOW_SIZE = "window_size"
+CONF_WRITE_PIN = "write_pin"
 CONF_X_GRID = "x_grid"
 CONF_Y_GRID = "y_grid"
 CONF_ZERO = "zero"
@@ -863,12 +880,14 @@ UNIT_MICROSIEMENS_PER_CENTIMETER = "µS/cm"
 UNIT_MICROTESLA = "µT"
 UNIT_MILLIGRAMS_PER_CUBIC_METER = "mg/m³"
 UNIT_MILLISECOND = "ms"
+UNIT_MILLISIEMENS_PER_CENTIMETER = "mS/cm"
 UNIT_MINUTE = "min"
 UNIT_OHM = "Ω"
 UNIT_PARTS_PER_BILLION = "ppb"
 UNIT_PARTS_PER_MILLION = "ppm"
 UNIT_PASCAL = "Pa"
 UNIT_PERCENT = "%"
+UNIT_PH = "pH"
 UNIT_PULSES = "pulses"
 UNIT_PULSES_PER_MINUTE = "pulses/min"
 UNIT_SECOND = "s"
@@ -889,7 +908,6 @@ DEVICE_CLASS_GARAGE_DOOR = "garage_door"
 DEVICE_CLASS_HEAT = "heat"
 DEVICE_CLASS_LIGHT = "light"
 DEVICE_CLASS_LOCK = "lock"
-DEVICE_CLASS_MOISTURE = "moisture"
 DEVICE_CLASS_MOTION = "motion"
 DEVICE_CLASS_MOVING = "moving"
 DEVICE_CLASS_OCCUPANCY = "occupancy"
@@ -907,15 +925,17 @@ DEVICE_CLASS_WINDOW = "window"
 # device classes of both binary_sensor and sensor component
 DEVICE_CLASS_EMPTY = ""
 DEVICE_CLASS_BATTERY = "battery"
+DEVICE_CLASS_CARBON_MONOXIDE = "carbon_monoxide"
 DEVICE_CLASS_GAS = "gas"
+DEVICE_CLASS_MOISTURE = "moisture"
 DEVICE_CLASS_POWER = "power"
 # device classes of sensor component
 DEVICE_CLASS_APPARENT_POWER = "apparent_power"
 DEVICE_CLASS_AQI = "aqi"
 DEVICE_CLASS_CARBON_DIOXIDE = "carbon_dioxide"
-DEVICE_CLASS_CARBON_MONOXIDE = "carbon_monoxide"
 DEVICE_CLASS_CURRENT = "current"
 DEVICE_CLASS_DATE = "date"
+DEVICE_CLASS_DISTANCE = "distance"
 DEVICE_CLASS_DURATION = "duration"
 DEVICE_CLASS_ENERGY = "energy"
 DEVICE_CLASS_FREQUENCY = "frequency"
@@ -930,14 +950,20 @@ DEVICE_CLASS_PM1 = "pm1"
 DEVICE_CLASS_PM10 = "pm10"
 DEVICE_CLASS_PM25 = "pm25"
 DEVICE_CLASS_POWER_FACTOR = "power_factor"
+DEVICE_CLASS_PRECIPITATION_INTENSITY = "precipitation_intensity"
 DEVICE_CLASS_PRESSURE = "pressure"
 DEVICE_CLASS_REACTIVE_POWER = "reactive_power"
 DEVICE_CLASS_SIGNAL_STRENGTH = "signal_strength"
+DEVICE_CLASS_SPEED = "speed"
 DEVICE_CLASS_SULPHUR_DIOXIDE = "sulphur_dioxide"
 DEVICE_CLASS_TEMPERATURE = "temperature"
 DEVICE_CLASS_TIMESTAMP = "timestamp"
 DEVICE_CLASS_VOLATILE_ORGANIC_COMPOUNDS = "volatile_organic_compounds"
 DEVICE_CLASS_VOLTAGE = "voltage"
+DEVICE_CLASS_VOLUME = "volume"
+DEVICE_CLASS_WATER = "water"
+DEVICE_CLASS_WIND_SPEED = "wind_speed"
+DEVICE_CLASS_WEIGHT = "weight"
 # device classes of both binary_sensor and button component
 DEVICE_CLASS_UPDATE = "update"
 # device classes of button component
